@@ -11,6 +11,8 @@ import minknow.rpc.manager_pb2
 import minknow.rpc.manager_pb2_grpc
 import minknow.rpc.protocol_pb2
 import minknow.rpc.protocol_pb2_grpc
+import minknow.rpc.acquisition_pb2
+import minknow.rpc.acquisition_pb2_grpc
 
 import pyminknow.config
 
@@ -22,12 +24,18 @@ This is a client to test a Nanopore gene sequencing device by using its gRPC int
 Some commands are run against the manager (minKNOW) and some against a device (a mini
 """
 
-USAGE = "python -m pyminknow.client"
+USAGE = """
+# Assuming we're connecting to the minKNOW, get the port to use for each device
+python -m pyminknow.client --list_devices
+python -m pyminknow.client --port 8012 --start_protocol sequencing/sequencing_MIN106_DNA:FLO-MIN106:SQK-LSK109:True
+"""
 
 
 class RpcClient:
     """
-    gRPC client
+    Client to send and receive Protocol Buffers objects as part of the minKNOW gRPC interface:
+
+    https://github.com/nanoporetech/minknow_lims_interface/tree/master/minknow/rpc
     """
 
     # TODO timeouts
@@ -71,6 +79,8 @@ class RpcClient:
 class ManagerClient(RpcClient):
     """
     Manager client
+
+    https://github.com/nanoporetech/minknow_lims_interface/blob/master/minknow/rpc/manager.proto
     """
 
     stub_name = 'manager'
@@ -99,7 +109,7 @@ class ProtocolClient(RpcClient):
 
     stub_name = 'protocol'
 
-    def list_protocols(self):
+    def list_protocols(self) -> minknow.rpc.protocol_pb2.ListProtocolsResponse:
         request = minknow.rpc.protocol_pb2.ListProtocolsRequest()
         return self.stub.list_protocols(request)
 
@@ -128,27 +138,35 @@ class ProtocolClient(RpcClient):
 
         return self.stub.start_protocol(request)
 
+    def stop_protocol(self, data_action_on_stop: int):
+        """
+        Stops the currently running protocol script instance
+
+        data_action_on_stop:
+          https://github.com/nanoporetech/minknow_lims_interface/blob/master/minknow/rpc/acquisition.proto#L260
+        """
+        # >>> minknow.rpc.acquisition_pb2.StopRequest.DataAction.items()
+        # [('STOP_DEFAULT', 0), ('STOP_KEEP_ALL_DATA', 1), ('STOP_FINISH_PROCESSING', 2)]
+        request = minknow.rpc.protocol_pb2.StopProtocolRequest(data_action_on_stop=data_action_on_stop)
+        return self.stub.stop_protocol(request)
+
     def set_sample_id(self, sample_id: str) -> minknow.rpc.protocol_pb2.SetSampleIdResponse:
         warnings.warn('The sample_id should be set in the request when a protocol starts ( start_protocol() )',
                       DeprecationWarning)
         request = minknow.rpc.protocol_pb2.SetSampleIdRequest(sample_id=sample_id)
         return self.stub.set_sample_id(request)
 
-    def list_protocol_runs(self) -> list:
+    def list_protocol_runs(self) -> minknow.rpc.protocol_pb2.ListProtocolRunsResponse:
         request = minknow.rpc.protocol_pb2.ListProtocolRunsRequest()
-        response = self.stub.list_protocol_runs(request)
-        return response.run_ids
+        return self.stub.list_protocol_runs(request)
 
     @property
-    def latest_run_id(self) -> int:
-        return self.list_protocol_runs()[0]
+    def latest_run_id(self) -> str:
+        return next(iter(self.list_protocol_runs().run_ids))
 
     def get_run_info(self, run_id: str = None) -> minknow.rpc.protocol_pb2.ProtocolRunInfo:
-        # Get latest run ID
-        if run_id is None:
-            run_id = self.latest_run_id
-
-        request = minknow.rpc.protocol_pb2.GetRunInfoRequest(run_id=run_id)
+        # If no run is specified, use the most recent one
+        request = minknow.rpc.protocol_pb2.GetRunInfoRequest(run_id=run_id or self.latest_run_id)
         return self.stub.get_run_info(request)
 
     def wait_for_finished(self, run_id: str, state: int = 0,
@@ -164,19 +182,22 @@ class ProtocolClient(RpcClient):
 class DeviceClient(RpcClient):
     """
     Device client
+
+    https://github.com/nanoporetech/minknow_lims_interface/blob/master/minknow/rpc/device.proto
     """
 
     stub_name = 'device'
 
     def get_device_state(self) -> minknow.rpc.device_pb2.GetDeviceStateResponse:
         request = minknow.rpc.device_pb2.GetDeviceStateRequest()
-        response = self.stub.get_device_state(request)
+        return self.stub.get_device_state(request)
 
-        # Get human-readable state
+    def get_device_state_name(self) -> str:
+        """Get human-readable state"""
+        response = self.get_device_state()
         state_name = minknow.rpc.device_pb2.GetDeviceStateResponse.DeviceState.Name(response.device_state)
-        LOGGER.info("Device status: %s => '%s'", response.device_state, state_name)
-
-        return response
+        LOGGER.debug("Device status: %s => '%s'", response.device_state, state_name)
+        return state_name
 
     def get_device_info(self) -> minknow.rpc.device_pb2.GetDeviceInfoResponse:
         request = minknow.rpc.device_pb2.GetDeviceInfoRequest()
@@ -185,7 +206,7 @@ class DeviceClient(RpcClient):
     def get_flow_cell_info(self) -> minknow.rpc.device_pb2.GetFlowCellInfoResponse:
         request = minknow.rpc.device_pb2.GetFlowCellInfoRequest()
         response = self.stub.get_flow_cell_info(request)
-        LOGGER.info("has_flow_cell: %s", response.has_flow_cell)
+        LOGGER.debug("has_flow_cell: %s", response.has_flow_cell)
         return response
 
 
@@ -210,20 +231,25 @@ def get_args():
                         help='List previously started protocol run IDs, in order of starting')
     parser.add_argument('-u', '--get_run_info', action='store_true', help="Get run info (Protocol)")
     parser.add_argument('-n', '--run_id', help="Protocol run identifier")
+    parser.add_argument('-a', '--sample_id', help="Sample identifier")
 
     return parser, parser.parse_args()
 
 
-def connect(host: str, port: int):
-    """
-    Connect to the server
-    """
+def connect(host: str = 'localhost', port: int = 9501):
+    """Connect to the server by opening a gRPC channel"""
 
     target = '{host}:{port}'.format(host=host, port=port)
 
     LOGGER.info("Connecting to '%s'...", target)
 
-    channel = grpc.insecure_channel(target=target)
+    # gRPC channel options
+    # https://grpc.github.io/grpc/core/group__grpc__arg__keys.html
+    options = [
+        # ('GRPC_ARG_SERVER_HANDSHAKE_TIMEOUT_MS', 10),
+    ]
+
+    channel = grpc.insecure_channel(target=target, options=options)
 
     return channel
 
@@ -265,9 +291,9 @@ def main():
 
             # Launch new protocol run
             elif args.start_protocol:
-                run_id = client.start_protocol(
+                response = client.start_protocol(
                     identifier=args.start_protocol,
-                    user_info=dict(protocol_group_id=args.protocol_group_id),
+                    user_info=dict(protocol_group_id=args.protocol_group_id, sample_id=args.sample_id),
                     args=[
                         "--fast5=on",
                         "--fast5_data", "trace_table", "fastq", "raw", "zlib_compress",
@@ -277,12 +303,12 @@ def main():
                         "--experiment_time=24"
                     ],
                 )
+                run_id = response.run_id
                 LOGGER.info("Started run ID: %s", run_id)
                 run_info = client.get_run_info(run_id)
                 print(run_info)
             elif args.list_protocol_runs:
-                run_ids = client.list_protocol_runs()
-                print(run_ids)
+                print(client.list_protocol_runs())
             else:
                 raise ValueError('Unknown command')
 
